#ifndef FUSION_H
#define FUSION_H

#include <string>
#include <vector>

#include "GridMap.h"
#include "Localization.h"

using namespace std;

// CFusion
/** 複数の距離センサーデータから、存在確率マップを生成するクラス
 */
class CFusion
{
public:
	/** コンストラクタ
	 */
	CFusion();

	/** デストラクタ
	 */
	virtual ~CFusion();

	/** 初期化処理
	 *
	 * @return 常に0を返す
	 */
	int init();

	/** メモリパラメータ設定
	 *
	 * @param[in] period データ保持時間設定
	 * @param[in] decrease 時間経過によるデータ減衰率
	 * @return    常に0を返す.
	 */
	int setMemoryPara(float period, float decrease);
	
	/** 統合用データにデータを追加する
	 *
	 * 統合するデータに要素を一つ追加する.
	 * 追加すると同時に, データ保持時間を過ぎたデータを削除する.
	 *
	 * @param[in] sensor_data 追加する存在確率マップ
	 * @return    常に0を返す.
	 */
	int setGridMap(GridMap *sensor_data);
	
	/** 作業用マップを設定する
	 *
	 * @param[in] grid_map 作業用マップ
	 * @return    常に0を返す.
	 */
	int setWorkMap(GridMap *grid_map);

	/** ロボット自己位置を登録する
	 *
	 * @param[in] self_loc ロボット自己位置情報
	 * @return    常に0を返す.
	 */
	int setSelfLocalization(Localization *self_loc);

	/** 統合されたデータを受け取る.
	 *
	 * @param[out] result 統合されたデータを取得するポインタ
	 * @return     常に0を返す.
	 */
	int getMergedGridMap(GridMap *result); // 統合したセンサ値のポインタ

	/** 特定障害物を取得する？
	 *
	 * 未実装
	 *
	 * @param[out] detect_object 特定障害物を受け取るポインタ？
	 * @return     常に0を返す？
	 */
	int getDetectedObject(struct detect_object_T *detect_object); // 特定障害物

	/** 終了処理を行う
	 *
	 * @return 常に0を返す.
	 */
	int close();

private:
	/** 自己位置を更新する
	 *
	 * 保持しているセンサデータの中から最新のものを探し, その原点を自己位置に設定する
	 *
	 * @param[out] self_loc 更新する自己位置
	 * @param[in]  time 自己位置の前回更新時刻
	 * @return     データ更新時は0を返す, それ以外は-1を返す.
	 */
	int getEstimatedLocalization(Localization *self_loc, double time);

	/** 存在確率マップを変形する
	 *
	 * 存在確率マップの変形 (移動, 回転, 拡大縮小) を行う.
	 *
	 * @param[in] src 変形元の存在確率マップ
	 * @param[in] dx 移動量 (X軸)
	 * @param[in] dy 移動量 (y軸)
	 * @param[in] dthe 回転角度
	 * @param[in] dst 変形後の存在確率マップを受け取るポインタ(こちらの座標系に変換する)
	 * @return 常に0を返す
	 */
	int transform(GridMap *src, float dx, float dy, float dthe, GridMap *dst);

	float period; ///< データの保持時間
	float decrease; ///< 時間経過によるデータの減衰率

	vector<GridMap> stored_data; ///< 保持しているセンサデータ履歴
	vector<Localization> stored_self_loc; ///< 自己位置履歴

	GridMap *work_map; ///<作業用マップ
};

#endif // FUSION_H
